-- create lexer with "alex -g MapLexer.x"
{

{-
Part of "Haski the Robot"
examination project for adv. funct. progr. 2004

    Copyright (C) 2004  Lars Otten

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-}

{-

  I M P O R T A N T  ! ! !
----------------------------
after creating the lexer with Alex one has to the replace the
alexScanTokens-function with the following:

alexScanTokens str = go (alexStartPos,'\n',str)
  where go inp@(pos,_,str) =
	  case alexScan inp 0 of
		AlexEOF -> return []
		AlexError (p,c,s) -> failE ("lexical error in line "
		                             ++ getLineNo p ++ ", column " 
		                             ++ getColNo p)
		AlexSkip  inp' len     -> go inp'
		AlexToken inp' len act -> do let x = act pos (take len str)
                                             y <- go inp'
                                             return (x:y)


-}

module MapLexer where

import ErrorMonad

}

%wrapper "posn"

$digit = 0-9
$alpha = [a-zA-Z]

tokens :-

  $white+				;
  "--".*				;
  ";"					{ \p s -> T p s TBREAK }
  ","					{ \p s -> T p s TComma }
  "R"					{ \p s -> T p s TRobot }
  "I"					{ \p s -> T p s TItem }
  $digit+				{ \p s -> T p s (TInt (read s)) }
  "("					{ \p s -> T p s TBO } -- bracket open
  ")"					{ \p s -> T p s TBC } -- bracket close

{

data TValue
  = TBREAK
  | TComma
  | TBO
  | TBC
  | TRobot
  | TItem
  | TInt Int
       deriving (Eq,Show)

-- saves the position info, the actual string
-- and the parsed Token
data MToken = T AlexPosn String TValue

-- gets the line/column number from a AlexPn-object
getLineNo (AlexPn _ l _) = show l
getColNo (AlexPn _ _ c) = show c

-- main function
lexer s = alexScanTokens s



}
